/*  descartes
 *  Copyright (C) 2001 Luc Brun
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include "info_region.h"
#include "gtk_info_region.h"
#include "geometrie.h"
#include "topologie.h"
#include "photometrie.h"
#include <glib.h>
#include <gnome.h>
#include <math.h>
#include "support.h"
#include "color_axis.h"
#include "histo1d.h"

typedef gchar* (*info_function)(contour_set,dart);

/* Intermediate variable used to fill the list */
static char value[50];

static gchar* info_geo_ppt[]=
{ 
  "Surface",
  "Perimeter",
  "Mean",
  "Points of Including Rectangle (min & max)",
  "Height of Including Rectangle",
  "Width of Including Rectangle",
  "Center of Including Rectangle",
  "Ratio Surface/(Height*Width)",
  "Ratio Perimeter/(2*Height+2*Width)",
  "Ratio (16*Surface)/(Perimeter**2)",
  "Log10(Height/Width)",
  "Direction of the Major Axis",
  "Angle of the Major Axis (radian)",
  "Angle of the Major Axis (degree)",
  "Elongation (Length/Width)",
  "Elongation (with the eigen values)"
};
static gchar* info_photo_ppt[]=
{
  "Mean Color",
  "Red Variance",
  "Green Variance",
  "Blue Variance",
  "Squared Error",
  "ExColor",
  "Dominant Color",
  "Intensity",
  "Hue",
  "Saturation",
  "First Color Vector",
  "Second Color Vector",
  "Third Color Vector",
  "Principal Axis Infos",
  "Principal Plane Infos"
};
static gchar* info_topo_ppt[]=
{
  "Number of neighbors",
  "Adjacent to an infinite face",
  "Number of included connected components",
  "Number of holes"
};

static int nb_ppt[3]={16,15,4};  

static gchar* unknown_priv           (contour_set,dart);
static gchar* surface_priv           (contour_set contours,dart f);
static gchar* perimetre_priv         (contour_set contours,dart f);
static gchar* mean_priv              (contour_set,dart);
static gchar* points_priv            (contour_set,dart);
static gchar* height_priv	     (contour_set,dart);
static gchar* width_priv	     (contour_set,dart);
static gchar* center_priv            (contour_set,dart);
static gchar* ratio_surface_priv     (contour_set,dart);
static gchar* ratio_perimetre_priv   (contour_set,dart);
static gchar* ratio_pixel_priv       (contour_set,dart);
static gchar* log_priv               (contour_set,dart);
static gchar* direction_priv         (contour_set,dart);
static gchar* angle_rad_priv         (contour_set,dart);
static gchar* angle_deg_priv         (contour_set,dart);
static gchar* elongation_priv        (contour_set,dart);
static gchar* elongation_bis_priv    (contour_set,dart);
static gchar* max_dist_priv          (contour_set,dart);

static gchar* mean_color_priv        (contour_set contours,dart f);
static gchar* red_variance_priv      (contour_set contours,dart f);
static gchar* green_variance_priv    (contour_set contours,dart f);
static gchar* blue_variance_priv     (contour_set contours,dart f);
static gchar* squared_error_priv     (contour_set contours,dart f);
static gchar* excolor_priv           (contour_set,dart);
static gchar* dominant_color_priv    (contour_set,dart);
static gchar* intensity_priv         (contour_set,dart);
static gchar* hue_priv               (contour_set,dart);
static gchar* saturation_priv        (contour_set,dart);
static gchar* color_vect_1_priv      (contour_set,dart);
static gchar* color_vect_2_priv      (contour_set,dart);
static gchar* color_vect_3_priv      (contour_set,dart);
static gchar* color_axis_priv        (contour_set,dart);
static gchar* color_plane_priv       (contour_set,dart);

static gchar* nb_neighbour_priv      (contour_set contours,dart f);
static gchar* infinite_neighbour_priv(contour_set contours,dart f);
static gchar* nb_included_faces_priv (contour_set contours,dart f);
static gchar* nb_hole_priv           (contour_set,dart);   

static info_function info_geo_value[]=
{
  surface_priv,
  perimetre_priv,
  mean_priv,
  points_priv,
  height_priv,
  width_priv,
  center_priv,
  ratio_surface_priv,
  ratio_perimetre_priv,
  ratio_pixel_priv,
  log_priv,
  direction_priv,
  angle_rad_priv,
  angle_deg_priv,
  elongation_priv,
  elongation_bis_priv
  
};
static info_function info_photo_value[]=
{
  mean_color_priv,
  red_variance_priv,
  green_variance_priv,
  blue_variance_priv,
  squared_error_priv,
  excolor_priv,
  dominant_color_priv,
  intensity_priv,
  hue_priv,
  saturation_priv,
  color_vect_1_priv,
  color_vect_2_priv,
  color_vect_3_priv,
  color_axis_priv,
  color_plane_priv
};
static info_function info_topo_value[]=
{
  nb_neighbour_priv,
  infinite_neighbour_priv,
  nb_included_faces_priv,
  nb_hole_priv
};

int create_info_region(contour_set contours,dart f)
{
  int i,j;
  GtkWidget* main_widget;
  static gchar** tab_name[]={info_geo_ppt,info_photo_ppt,info_topo_ppt};
  static info_function* get_value[]={info_geo_value,info_photo_value,info_topo_value};
  static gchar* text[2];
  static gtk_info_region_type_list list[3]=
  {geometrical_list,photometrical_list,topological_list};

  text[1]="Not yet available";

  if (contours==NULL)
    return -1;

  if (!contour_set_is_face_defined(contours,f))
    return -2;

  if (contour_set_is_infinite(contours,f))
    return -3;
  

  main_widget = create_gtk_info_region();

  for(i=0;i<3;i++)
    {

      for(j=0;j<nb_ppt[i];j++)	
	{
	  text[0]=tab_name[i][j];
	  text[1]= get_value[i][j](contours,f);
	  gtk_info_region_add_property(main_widget,
				       text,
				       list[i]);
	}
    }
  gtk_widget_show(main_widget);
  
  return 0;
}

static gchar* unknown_priv(contour_set contours,dart f)
{
  return "Not yet available";
}

static gchar* surface_priv(contour_set contours,
			   dart f)
{
  sprintf(value,"%d pixels",(int)surface_calc(contours,f));
  
  return value;
}

static gchar* perimetre_priv(contour_set contours,
			     dart f)
{
  sprintf(value,"%d",(int)perimetre_calc(contours,f));

  return value;
}

static gchar* mean_priv(contour_set contours,dart f)
{
  int x,y;

  mean_calc(contours,f,&x,&y);
  
  sprintf(value,"(%d,%d)",x,y);
  
  return value;
}

static gchar* points_priv(contour_set contours,dart f)
{
  int x1,y1,x2,y2;
  
  points_calc(contours,f,&x1,&y1,&x2,&y2);
  x1++;y1++;x2++;y2++;
  sprintf(value,"{(%d,%d),(%d,%d)}",x1,y1,x2,y2);
  
  return value;
}

static gchar* height_priv(contour_set contours,dart f)
{
  sprintf(value,"%d",(int)height_calc(contours,f));

  return value;
}

static gchar* width_priv(contour_set contours,dart f)
{
  sprintf(value,"%d",(int)width_calc(contours,f));

  return value;
}

static gchar* center_priv(contour_set contours,dart f)
{
  int x,y;
  
  center_calc(contours,f,&x,&y);
  
  sprintf(value,"(%d,%d)",x,y);
  
  return value;
}

static gchar* ratio_surface_priv(contour_set contours,dart f)
{
  sprintf(value,"%.2f",(double)ratio_surface_calc(contours,f));

  return value;
}

static gchar* ratio_perimetre_priv(contour_set contours,dart f)
{
  sprintf(value,"%.2f",(double)ratio_perimetre_calc(contours,f));

  return value;
}

static gchar* ratio_pixel_priv(contour_set contours,dart f)
{
  sprintf(value,"%.2f",(double)ratio_pixel_calc(contours,f));

  return value;
}

static gchar* log_priv(contour_set contours,dart f)
{
  sprintf(value,"%.2f",(double)log_calc(contours,f));
  
  return value;
}

static gchar* direction_priv(contour_set contours,dart f)
{
  double v1[2];

  direction_calc(contours,f,v1);

  sprintf(value,"(%.2f,%.2f)",v1[0],v1[1]);
  
  return value;
}

static gchar* angle_rad_priv(contour_set contours,dart f)
{
  double a1,a2;
  
  sprintf(value,"%.2f",(double)angle_calc(contours,f,&a1,&a2));

  return value;
}

static gchar* angle_deg_priv(contour_set contours,dart f)
{
  double res,a1,a2;

  res=(double)(180*angle_calc(contours,f,&a1,&a2)/acos(-1));

  sprintf(value,"%.2f",res);

  return value;
}

static gchar* elongation_priv(contour_set contours,dart f)
{
  sprintf(value,"%.2f",(double)elongation_calc(contours,f));

  return value;
}

static gchar* elongation_bis_priv(contour_set contours,dart f)
{
  sprintf(value,"%.2f",(double)elongation_bis_calc(contours,f));

  return value;
}

static gchar* max_dist_priv(contour_set contours,dart f)
{
  sprintf(value,"%.2f",(double)max_dist_calc(contours,f));
  
  return value;
}

static gchar* mean_color_priv(contour_set contours,
			      dart f)
{
  vector V;

  mean_color_calc(contours,f,V);
  
  sprintf(value,"(%.2f,%.2f,%.2f)",V[0],V[1],V[2]);
  
  return value;
}

static gchar* red_variance_priv(contour_set contours,
				dart f)
{
  sprintf(value,"%.2f",(double)red_variance_calc(contours,f));

  return value;
}
			     
static gchar* green_variance_priv(contour_set contours,
				  dart f)
{
  sprintf(value,"%.2f",(double)green_variance_calc(contours,f));

  return value;
}

static gchar* blue_variance_priv(contour_set contours,
				dart f)
{
  sprintf(value,"%.2f",(double)blue_variance_calc(contours,f));
  
  return value;
}

static gchar* squared_error_priv(contour_set contours,
				 dart f)
{
  sprintf(value,"%.2f",(double)squared_error_calc(contours,f));
  
  return value;
}

static gchar* excolor_priv(contour_set contours,dart f)
{
  vector V;
  
  excolor_calc(contours,f,V);
  
  sprintf(value,"(%.2f,%.2f,%.2f)",V[0],V[1],V[2]);
  
  return value;
}

static gchar* dominant_color_priv(contour_set contours,dart f)
{
  static char rouge[4]={'R','e','d','\0'};
  static char vert[6]={'G','r','e','e','n','\0'};
  static char bleu[5]={'B','l','u','e','\0'};
  static char aucune[5]={'N','o','n','e','\0'};

  int color;

  color = dominant_color_calc(contours,f);

  if(color==0) return rouge;
  if(color==1) return vert;
  if(color==2) return bleu;
  return aucune;
}

static gchar* intensity_priv(contour_set contours,dart f)
{
  sprintf(value,"%.2f",(double)intensity_calc(contours,f));

  return value;
}

static gchar* hue_priv(contour_set contours,dart f)
{
  double h;

  h=hue_calc(contours,f);

  sprintf(value,"%.2f",h);
  
  return value;
}

static gchar* saturation_priv(contour_set contours,dart f)
{
  double s;

  s=saturation_calc(contours,f);

  sprintf(value,"%.2f",s);
  
  return value;
}

static gchar* color_vect_1_priv(contour_set contours,dart f)
{
  static char nul[5]={'N','o','n','e','\0'};

  vecteur val;
  mat_passage vect;

  color_axis_calc(contours,f,val,vect);
  
  if((vect[0][0] == 0) && (vect[0][1] == 0) && (vect[0][2] == 0))
    return nul;
  else
    {
      sprintf(value,"(%.2f,%.2f,%.2f)",vect[0][0],vect[0][1],vect[0][2]);
      
      return value;
    }
}

static gchar* color_vect_2_priv(contour_set contours,dart f)
{
  static char nul[5]={'N','o','n','e','\0'};

  vecteur val;
  mat_passage vect;

  color_axis_calc(contours,f,val,vect);

  if((vect[1][0] == 0) && (vect[1][1] == 0) && (vect[1][2] == 0))
    return nul;
  else
    {
      sprintf(value,"(%.2f,%.2f,%.2f)",vect[1][0],vect[1][1],vect[1][2]);
      
      return value;
    }
}

static gchar* color_vect_3_priv(contour_set contours,dart f)
{
  static char nul[5]={'N','o','n','e','\0'};

  vecteur val;
  mat_passage vect;

  color_axis_calc(contours,f,val,vect);

  if((vect[2][0] == 0) && (vect[2][1] == 0) && (vect[2][2] == 0))
    return nul;
  else
    {
      sprintf(value,"(%.2f,%.2f,%.2f)",vect[2][0],vect[2][1],vect[2][2]);
      
      return value;
    }
}

static gchar* color_axis_priv(contour_set contours,dart f)
{
  static char nul[5]={'N','o','n','e','\0'};

  double axe,plan;
  
  color_axis_plane_calc(contours,f,&axe,&plan);
  
  if(axe == -1) return nul;
  else
    {
      sprintf(value,"%.2f",axe);
      
      return value;
    }
}

static gchar* color_plane_priv(contour_set contours,dart f)
{
  static char nul[5]={'N','o','n','e','\0'};

  double axe,plan;
  
  color_axis_plane_calc(contours,f,&axe,&plan);

  if(plan == -1) return nul;
  else
    {
      sprintf(value,"%.2f",plan);

      return value;
    }
}

static gchar* nb_neighbour_priv(contour_set contours,
				dart f)
{
  sprintf(value,"%d",(int)nb_neighbour_calc(contours,f));

  return value;
}

static gchar* infinite_neighbour_priv(contour_set contours,
				       dart f)
{
  static char oui[4]={'y','e','s','\0'};
  static char non[4]={'n','o',' ','\0'};

  if(infinite_neighbour_calc) return oui;
  return non;
}

static gchar* nb_included_faces_priv(contour_set contours,
				    dart f)
{
  sprintf(value,"%d",(int)nb_included_faces_calc(contours,f));

  return value;
}

static gchar* nb_hole_priv(contour_set contours,dart f)
{
  sprintf(value,"%d",(int)nb_hole_calc(contours,f));

  return value;
}
