/*  descartes
 *  Copyright (C) 2001 Luc Brun
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include "move_type.h"
#include "objet.h"
#include "chemin_discret.h"
#include "droite.h"


/* TYPE DROITE DISCRETE : mu <= ax - by < mu + b */

#define A        SELF(a)
#define B        SELF(b)
#define Mu       SELF(mu)
#define Quadrant SELF(quadrant)

struct Droite_Discrete{
    int a;
    int b;
    int mu;
    int quadrant;
};


static int correspondance_codes_quadrant[4][4] = {{0,0,-1,3}, {0,1,1,-1},
						  {-1,1,2,2}, {3,-1,2,3}};


static void set_caract_priv(Droite_Discrete, int, int, int);
static void affect_droite_priv(Droite_Discrete, Droite_Discrete);
static int ajouter_point_priv(Droite_Discrete, int, int); 
static int ajouter_point_positif_priv(Droite_Discrete, int, int, int, int);
static int ajouter_point_negatif_priv(Droite_Discrete, int, int, int, int);
static void reconnaitre_droite_priv(Droite_Discrete, Chemin_Discret,
				    int, move_type, move_type);
static int variation_y_priv(Chemin_Discret, int, int, move_type, move_type); 
static int repere_quadrant_priv(Chemin_Discret, int, move_type*, move_type*);

/* CREATION - LIBERATION - REINITIALISATION */

Droite_Discrete FAIRE_Droite_Discrete(){
  INSTANCIER(Droite_Discrete);
  A = 0;
  B = 1;
  Mu = 0;
  Quadrant = -1;
  return self;
}

void DEFAIRE_Droite_Discrete(Droite_Discrete self){
  free(self);
}

void Droite_Discrete_reinit(Droite_Discrete self){
  A = 0;
  B = 1;
  Mu = 0;
  Quadrant = -1;
}


/* ACCES AUX CARACTERISTIQUES */

int Droite_Discrete_give_a(Droite_Discrete self){
  return A;
}

int Droite_Discrete_give_b(Droite_Discrete self){
  return B;
}

int Droite_Discrete_give_mu(Droite_Discrete self){
  return Mu;
}

int Droite_Discrete_give_quadrant(Droite_Discrete self){
  return Quadrant;
}


/* CALCUL DE LA TANGENTE LOCALE EN UN PT D'UN CHEMIN DISCRET */

void Droite_Discrete_calcul_tangente(Droite_Discrete self,
				    Chemin_Discret chem, 
				    int ind_pt){
  int quadrant;
  move_type code1, code2;

  quadrant = repere_quadrant_priv(chem, ind_pt, &code1, &code2);
  reconnaitre_droite_priv(self, chem, ind_pt, code1, code2);
  Quadrant = quadrant;
}


/* FONCTIONS INTERMEDIAIRES */

static void set_caract_priv(Droite_Discrete self, int a, int b, int mu){
  A = a;
  B = b;
  Mu = mu;
}

static void affect_droite_priv(Droite_Discrete self, Droite_Discrete d){
  A = d->a;
  B = d->b;
  Mu = d->mu;
}


/***********************************************************************
* DONNE LES CARACTERISTIQUES D'UNE DROITE A PARTIR DE SA DISCRETISATION*
*      METHODE DEBLED geometrie discrete Grenoble 92                   *
************************************************************************/

static int ajouter_point_priv(Droite_Discrete self, int x, int y){ 
  int reste, k, l;

  reste = A*x - B*y;
  if(Mu <= reste && reste < Mu + B)
    return 1;

  else if(reste == Mu - 1){ /* la pente augmente */
    k = 0;
    while((A*k - Mu)%B != 0)
      k++;
    l = (A*k - Mu)/B;
    A = y-l;
    B = x-k;
    Mu = A*x - B*y;
    return 1;
  }

  else if(reste == Mu + B){ /* la pente diminue */
    k = 0;
    while((A*k - Mu - B + 1)%B != 0)
      k++;
    l = (A*k - Mu)/B;
    A = y-l;
    B = x-k;
    Mu = A*x - B*(y+1) + 1;
    return 1;
  }
  else /* le nouveau point ne permet pas de continuer la droite */
    return 0;
}


static int ajouter_point_positif_priv(Droite_Discrete self, 
				      int x, int y, 
				      int xO, int yO){
  int X, Y, ajout_ok;
  
  X = x - xO;  /* translation */
  Y = y - yO;

  Mu = Mu - A*xO + B*yO;
  ajout_ok = ajouter_point_priv(self, X, Y);
  Mu = Mu + A*xO - B*yO;
  return ajout_ok;
}

static int ajouter_point_negatif_priv(Droite_Discrete self, 
				      int x, int y, 
				      int xO, int yO){
  int X, Y, ajout_ok;
    
  X = -x + xO; /* translation , sym axe x, sym axe y */
  Y = -y + yO;
  
  Mu = 1 - B - Mu + A*xO - B*yO;
  ajout_ok = ajouter_point_priv(self, X, Y);
  Mu = 1 - B - Mu + A*xO - B*yO;
  return ajout_ok;
}


static void reconnaitre_droite_priv(Droite_Discrete self,
				    Chemin_Discret chem,
				    int ind_pt,
				    move_type code1, move_type code2){
  int ok = 1;
  int i;
  int x_plus, y_plus, x_moins, y_moins, 
  x_plus_save, y_plus_save, x_moins_save, y_moins_save;
  int code_plus, code_moins;
  Droite_Discrete d_save;
  
  d_save = FAIRE_Droite_Discrete();

  /*initialisations */
  x_plus = 1;
  if(variation_y_priv(chem, ind_pt, 1, code1, code2) == 0) /* pente nulle */
    y_plus = 0;
  else {            /*pente = 1 */
    y_plus = 1;
    A = 1;
  }
  
  x_moins = -1;
  if(variation_y_priv(chem, ind_pt, -1, code1, code2) == 0)
    y_moins = 0;
  else
    y_moins = -1;

  ok = ajouter_point_negatif_priv(self, x_moins, y_moins, x_plus, y_plus);
  
  i = 2;
  while(ok){
    affect_droite_priv(d_save,self);
    x_plus_save = x_plus;
    y_plus_save = y_plus;
    x_moins_save = x_moins;
    y_moins_save = y_moins;
    code_plus = variation_y_priv(chem, ind_pt, i, code1, code2);
    if(code_plus != -1){
      x_plus++;
      if(code_plus == 1)
	y_plus++;
      ok = ajouter_point_positif_priv(self, 
				      x_plus, y_plus, 
				      x_moins, y_moins);
    }
    else
      ok = 0;
    
    code_moins = variation_y_priv(chem, ind_pt, -i, code1, code2);
    if(ok && (code_moins != -1)){
      x_moins--;
      if(code_moins == 1)
	y_moins--;
      ok = ajouter_point_negatif_priv(self, 
				      x_moins, y_moins, 
				      x_plus, y_plus);
    }
    else 
      ok = 0;

    i++;
  }

  i--;
  ok = 1;
  affect_droite_priv(self, d_save);
  x_plus = x_plus_save;
  y_plus = y_plus_save;
  x_moins = x_moins_save;
  y_moins = y_moins_save;
  if(code_plus == -1){
    while(ok && code_moins != -1){
      x_moins--;
      if(code_moins == 1)
	y_moins--;
      ok = ajouter_point_negatif_priv(self, 
				      x_moins, y_moins, 
				      x_plus, y_plus);
      i++;
      code_moins = variation_y_priv(chem, ind_pt, -i, code1, code2);
    }
  }

  else if(code_moins == -1){
    while(ok && code_plus != -1){
      x_plus++;
      if(code_plus == 1)
	y_plus++;
      ok = ajouter_point_positif_priv(self, 
				      x_plus, y_plus, 
				      x_moins, y_moins);
      i++;
      code_plus = variation_y_priv(chem, ind_pt, i, code1, code2);
    }
  }
  DEFAIRE_Droite_Discrete(d_save);
}



/* VARIATIONS DE Y CORRESPONDANT A 1 CHEMIN 1/2 RAMENE DS LE 1er QUADRANT*/

static int variation_y_priv(Chemin_Discret chem, 
			    int ind_pt, int decal,
			    move_type code1, move_type code2){
  move_type elt_code;
  
  elt_code = Chemin_Discret_code_decalage_circulaire(chem, ind_pt, decal);

  if(elt_code != code1 && elt_code != code2)
    return -1;

  if(TEST_HORIZONTAL(elt_code))
    return 0;
  else
    return 1;
}




/* CALCUL DU VOISINAGE D'UN POINT 1/2  */

static int repere_quadrant_priv(Chemin_Discret chem, int ind_pt, 
				move_type* code1, move_type* code2){ 
  int i; 
  move_type c, c1, c2;
  
  c1 = Chemin_Discret_code_decalage_circulaire(chem, ind_pt, 1);
  c2 = Chemin_Discret_code_decalage_circulaire(chem, ind_pt, -1);

  if(c1 != c2){
    *code1 = c1;
    *code2 = c2;
  }
  else{ /* on doit chercher plus loin */
    c = c1;
    i = 2;
    c1= Chemin_Discret_code_decalage_circulaire(chem, ind_pt, i);
    c2= Chemin_Discret_code_decalage_circulaire(chem, ind_pt, -i);
    
    while(c1 == c && c2 == c){
      i++;
      c1= Chemin_Discret_code_decalage_circulaire(chem, ind_pt, i);
      c2= Chemin_Discret_code_decalage_circulaire(chem, ind_pt, -i);
    }
    
    if(c1 != c && c2 != c && c1 != c2){   /* stop! */
      *code1 = c;
      *code2 = c;
    }
    else if (c1 == c2){
      *code1 = c;
      *code2 = c1;
    }
    else{ /* soit c1 soit c2 est egal a c */
      *code1 = c1;
      *code2 = c2;
    }
  }

  return correspondance_codes_quadrant[*code1][*code2];
}










