/*
 *  File:	hash.c
 *  Author:	Robert Strandh
 *  Purpose:
 *      Fash general hash tables.
 *
 *  $Id: hash.c,v 1.2 1992/03/19 21:22:57 david Exp $
 */
static char rcsid[] =
   "$Id: hash.c,v 1.2 1992/03/19 21:22:57 david Exp $";

#include "hash.h"

/* minimum hash table size in pointers */
#define NULL 0

#ifndef MIN_HASH_TABLE_SIZE
#define MIN_HASH_TABLE_SIZE 3
#endif

#ifndef MAX_FILL_DEGREE
#define MAX_FILL_DEGREE 5
#endif

#ifndef MIN_FILL_DEGREE
#define MIN_FILL_DEGREE 1
#endif


hash_table
hash_create(hash_anything (*malloc)(int size),
	    void (*free)(hash_anything),
	    int (*hash)(hash_object),
	    int (*equal)(hash_object,hash_object),
	    void (*deallocate)(hash_object))
{
    hash_table temp;
    int i;
    temp = (hash_table) (*malloc)(sizeof (struct hash_table_structure));
    temp->hash_table_size = MIN_HASH_TABLE_SIZE;
    temp->number_of_bindings = 0;
    temp->hash_table = (hash_binding *) (*malloc)(MIN_HASH_TABLE_SIZE *
						  sizeof(hash_binding));
    for(i = 0; i < MIN_HASH_TABLE_SIZE; i++)
	temp->hash_table[i] = NULL;
    temp->cursor = NULL;
    temp->malloc = malloc;
    temp->free = free;
    temp->equal = equal;
    temp->hash = hash;
    temp->deallocate = deallocate;

    return temp;
}

void hash_destroy(hash_table ht)
{
    hash_binding bind, temp;
    int i;
    for(i = 0; i < ht->hash_table_size; i++)
    {
	for(bind = ht->hash_table[i]; bind; )
	{
	    temp = bind->next;
	    (*(ht->deallocate))(bind->obj);
	    (*(ht->free))(bind);
	    bind = temp;
	}
    }
    (*(ht->free))(ht->hash_table);
    free(ht);
}

void hash_reinitialize(hash_table ht)
{
  hash_binding bind, temp;
  int i;

  for(i = 0; i < ht->hash_table_size; i++)
    {
      for(bind = ht->hash_table[i]; bind; )
	{
	  temp = bind->next;
	  (*(ht->deallocate))(bind->obj);
	  (*(ht->free))(bind);
	  bind = temp;
	}
      ht->hash_table[i] = NULL;
      }
  ht->number_of_bindings = 0;
  ht->cursor             = NULL;
}

int hash_position(hash_table ht, hash_object obj)
{
    int index = (*(ht->hash))(obj) % ht->hash_table_size;
    hash_binding *bind = &(ht->hash_table[index]);
    for(; *bind; bind = &((*bind)->next))
    {
	if((*(ht->equal))((*bind)->obj, obj))
	{
	    ht->cursor = bind;
	    return 1;
	}
    }
    ht->cursor = bind;
    return 0;
}


static void
resize_table(hash_table ht, int new_hash_table_size)
{
    int i;
    
    hash_binding *new_hash_table = (hash_binding *)
	(*(ht->malloc))(new_hash_table_size * sizeof(hash_binding));
    for(i = 0; i < new_hash_table_size; i++)
	new_hash_table[i] = NULL;
    for(i = 0; i < ht->hash_table_size; i++)
    {
	hash_binding bind, temp;
	for(bind = ht->hash_table[i]; bind; )
	{
	    int index = bind->hashval % new_hash_table_size;
	    temp = bind->next;
	    bind->next = new_hash_table[index];
	    new_hash_table[index] = bind;
	    bind = temp;
	}
    }
    (*(ht->free))(ht->hash_table);
    ht->hash_table = new_hash_table;
    ht->hash_table_size = new_hash_table_size;
}

static void
possibly_increase_size(hash_table ht)
{
    if (ht->number_of_bindings > ht->hash_table_size*MAX_FILL_DEGREE)
	resize_table(ht, (ht->hash_table_size) * 2 + 1); 
}

static void
    possibly_decrease_size(hash_table ht)
{
    if (ht->number_of_bindings < ht->hash_table_size*MIN_FILL_DEGREE
	&& ht->hash_table_size > MIN_HASH_TABLE_SIZE)
	resize_table(ht, ((ht->hash_table_size) - 1) / 2); 
}

void
hash_insert(hash_table ht, hash_object obj)
{
    hash_binding old = *(ht->cursor);
    (*(ht->cursor)) = (hash_binding)
	(*(ht->malloc))(sizeof(struct hash_binding_structure));
    (*(ht->cursor))->hashval = (*(ht->hash))(obj);
    (*(ht->cursor))->next = old;
    (*(ht->cursor))->obj = obj;
    (ht->number_of_bindings)++;
    possibly_increase_size(ht);
}

void
hash_delete(hash_table ht)
{
    hash_binding old = *(ht->cursor);
    *(ht->cursor) = old->next;
    (*(ht->deallocate))(old->obj);
    (*(ht->free))(old);
    (ht->number_of_bindings)--;
    possibly_decrease_size(ht);
}

void
hash_map(hash_table ht, void (*fun)(hash_object))
{
    int i;
    hash_binding bind;
    for(i = 0; i < ht->hash_table_size; i++)
    {
	for(bind = ht->hash_table[i]; bind; bind = bind->next)
	    (*fun)(bind->obj);
    }
}

hash_object
hash_get(hash_table ht)
{
    return (*(ht->cursor))->obj;
}
