/*
 *  File:	hash.h
 *  Author:     Robert Strandh
 *  Purpose:
 *      Headers for fast general hash tables.
 *
 *  $Id: hash.h,v 1.1 1992/03/19 18:31:26 david Exp $
 */

#ifndef HASH_H
#define HASH_H

typedef void *hash_anything;
typedef void *hash_object;

typedef struct hash_table_structure *hash_table;
typedef struct hash_binding_structure *hash_binding;

struct hash_table_structure
{
  int hash_table_size; 
  int number_of_bindings;
  hash_binding *hash_table;
  hash_binding *cursor;
  hash_anything (*malloc)(int size);
  void (*free)(hash_anything);
  int (*equal)(hash_object,hash_object);
  void (*deallocate)(hash_object);
  int (*hash)(hash_object);
  void (*print)(hash_object);
};

struct hash_binding_structure
{
  int hashval;
  hash_object obj;
  hash_binding next;
};



typedef hash_anything (*hash_malloc_type)(int size);
typedef void (*hash_free_type)(hash_anything);
typedef int (*hash_type)(hash_object);
typedef int (*hash_equal_type)(hash_object, hash_object);
typedef void (*hash_deallocate_type)(hash_object);

/* Create a hash table. Malloc() is a function that allocates a block of
   memory. Free(), frees memory that has been allocated by malloc().
   Hash() takes an object and returns a hash value. Equal() is
   a function that return 1 if object arguments are equal, 0 otherwise.
   Deallocate() is called whenever an object is removed from the hash
   table. */

extern hash_table
  hash_create(hash_anything (*Malloc)(int size), void (*Free)(hash_anything),
	    int (*hash)(hash_object), int (*equal)(hash_object,hash_object),
	    void (*deallocate)(hash_object));

/* destroy a hash table. The deallocate() function given at creation will 
   be called for each of the objects contained in the table. */

extern void hash_destroy(hash_table);

/* The hash table has the notion of a cursor. This function positions 
   the cursor to an object equal() to the one given as argument, if such
   an object exists. In that case a 1 is returned. If no object equal() 
   to the one given exists in the table, a 0 is returned, and the cursor
   is positioned to a place where insertion of the new object should
   take place. A typical call would be
   	
		if(!hash_position(t,o)) hash_insert(t,o);
   
   that is, first position to the plase where object o ought to be located.
   If it is not in the table, then cursor is positioned where it should be 
   inserted, thus you can immediately insert it.
*/
extern int hash_position(hash_table, hash_object);

/* Insert an object into a hash table. Before insertion, hash_position() must
   have been called and must have returned 0. Otherwise the result of the
   insertion is undefined. */

extern void hash_insert(hash_table, hash_object);

/* Delete an object from a hash table. Before deletion, hash_position() must
   have been called and must have returned a 1. Otherwise the result of the
   deletion is undefined. The deallocate() function given at table creation
   time is called, with the deleted object as argument. */

extern void hash_delete(hash_table);

/* Map any function over all the elements of the hash table. The function 
   is applied to each of the objects in some arbitrary order. */

extern void hash_map(hash_table, void (*fun)(hash_object));

/* Get an object from a hash table. Before this function is called,
   hash_position() must have been called and must have returned a 1.
   Otherwise the result of the call is undefined. */

extern hash_object hash_get(hash_table);


/* Reinitializethe hash structure. All objects are removed. Hash
structure becomes empty */

extern void hash_reinitialize(hash_table);
#endif







